-----------------------------------------------------------------------
--                              Mast                                 --
--     Modelling and Analysis Suite for Real-Time Applications       --
--                                                                   --
--                       Copyright (C) 2000-2009                     --
--                 Universidad de Cantabria, SPAIN                   --
--                                                                   --
-- Authors: Michael Gonzalez       mgh@unican.es                     --
--          Jose Javier Gutierrez  gutierjj@unican.es                --
--          Jose Carlos Palencia   palencij@unican.es                --
--          Jose Maria Drake       drakej@unican.es                  --
--          Maria Cue              cuem@unican.es                    --
--          Juan Maria Rivas	   rivasjm@unican.es                 --
--                                                                   --
-- This program is free software; you can redistribute it and/or     --
-- modify it under the terms of the GNU General Public               --
-- License as published by the Free Software Foundation; either      --
-- version 2 of the License, or (at your option) any later version.  --
--                                                                   --
-- This program is distributed in the hope that it will be useful,   --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of    --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU --
-- General Public License for more details.                          --
--                                                                   --
-- You should have received a copy of the GNU General Public         --
-- License along with this program; if not, write to the             --
-- Free Software Foundation, Inc., 59 Temple Place - Suite 330,      --
-- Boston, MA 02111-1307, USA.                                       --
--                                                                   --
-----------------------------------------------------------------------

with Mast.Processing_Resources,
     Mast.HOSPA_Parameters,
     Mast.Annealing_Parameters,
     Mast.Linear_Translation,
     Priority_Queues,Hash_Lists,List_Exceptions,
     Mast.Tools.Schedulability_Index,
     Mast.Scheduling_Parameters,
     Mast.Scheduling_Policies,Mast.Tools;

with Ada.Text_IO,Ada.Integer_Text_IO,
     Ada.Numerics.Discrete_Random,Ada.Numerics.Float_Random,
     Ada.Numerics.Generic_Elementary_Functions,
     Mast.Max_Numbers,Ada.Float_Text_IO;

use type Mast.Tools.Schedulability_Index.Index;
use Ada.Text_IO,Ada.Integer_Text_IO,Ada.Float_Text_IO;

package body mast.linear_local_deadline_assignment_tools is

   ATC_Enabled : constant Boolean:=False;
   Debug : constant Boolean := False;
   My_Verbose : constant Boolean := False;

   type Processor_ID is new Natural;
   type Transaction_ID is new Natural;
   type Task_ID is new Natural;

   -----------------
   -- Local_HOSDA --
   -----------------

   procedure Local_HOSDA
     (The_System : in out Mast.Systems.System;
      The_Tool: in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose    : in Boolean:=False) is

      Optimize : Boolean;

      K : Hospa_Parameters.K_Pair;
      Ka, Kr: Hospa_Parameters.K_Type;
      -- Heuristic variables of the algorithm

      Analysis_Stop_Time : Duration := Hospa_Parameters.Get_Analysis_Stop_Time;

      Max_Iter : Integer; -- Heuristic number of iterations for eack K
      Schedulable, Overall_Schedulable : Boolean:=False;

      Counter_Of_Iterations : Integer:=0;

      Counter_Of_Aborted_Analysis : Integer := 0;

      Optimizing : Boolean := False;
      Overiteration : Integer;
      Stop_Algorithm : Boolean;
      Optimum_Sched_Index, Sched_Index :
      Mast.Tools.Schedulability_Index.Index:=
        Mast.Tools.Schedulability_Index.Lower_Index;

      Max_Processors:constant Processor_ID:=Processor_ID
        (Processing_Resources.Lists.Size
           (The_System.Processing_Resources));
      Max_Transactions:constant Transaction_ID:=Transaction_ID
        ((Mast.Max_Numbers.Calculate_Max_Transactions(The_System)));
      Max_Tasks_Per_Transaction:constant Task_ID:=Task_ID
        (Mast.Max_Numbers.Calculate_Max_Tasks_Per_Transaction(The_System));

      subtype Processor_ID_Type is Processor_ID range 0..Max_Processors;
      subtype Transaction_ID_Type is Transaction_ID range 0..Max_Transactions;
      subtype Task_ID_Type is Task_ID range 0..Max_Tasks_Per_Transaction;

      package Translation is new Linear_Translation
        (Processor_ID_Type, Transaction_ID_Type, Task_ID_Type,
         Max_Processors, Max_Transactions, Max_Tasks_Per_Transaction);

      use Translation;

      My_System,My_Back_Up_System : Translation.Linear_Transaction_System;

      type List_For_Tasks is array
        (Task_ID_Type range 1..Max_Tasks_Per_Transaction)
      of Translation.Priority_Assignment_Vars;

      type List_For_Transactions is array
        (Transaction_ID_Type range 1..Max_Transactions)
      of List_For_Tasks;

      Pavs : List_For_Transactions;


      --------------------
      -- Save Deadlines --
      --------------------

      procedure Save_Deadlines is
      begin
         for I in Transaction_ID_Type range 1..Max_Transactions
         loop
            for J in Task_ID_Type range 1..Max_Tasks_Per_Transaction
            loop
               My_System(I).The_Task(J).SDij :=
                 My_System(I).The_Task(J).Pav.D_0;
            end loop;
         end loop;
      end Save_Deadlines;

      ---------------
      -- Save_Pavs --
      ---------------

      procedure Save_Pavs is
      begin
         for I in Transaction_ID_Type range 1..Max_Transactions
         loop
            for J in Task_ID_Type range 1..Max_Tasks_Per_Transaction
            loop
               Pavs(I)(J) := My_System(I).The_Task(J).Pav;
            end loop;
         end loop;
      end Save_Pavs;

      ------------------
      -- Restore_Pavs --
      ------------------

      procedure Restore_Pavs is
      begin
         for I in Transaction_ID_Type range 1..Max_Transactions
         loop
            for J in Task_ID_Type range 1..Max_Tasks_Per_Transaction
            loop
               My_System(I).The_Task(J).Pav := Pavs(I)(J);
            end loop;
         end loop;
      end Restore_Pavs;



      --------------------------
      -- Initialize_Deadlines --
      --------------------------

      procedure Initialize_Deadlines is

         -- Initialize deadlines according to WCETs

         Max_Factor : constant Time := Large_Time;
         Max_D,Sum_C,Sum_D,Assigned_D,Factor : Time;

      begin
         Translation.Translate_Linear_System(The_System,My_System,My_Verbose);

         -- Deadline assignment

         declare
            I : Transaction_Id_Type;
            Extra : Transaction_Id_Type;
            -- Number of extra consecutive transactions with the same ID
         begin

            I := 1;
            while My_System(I).Ni/=0
            loop

               -- Calculate Extra
               Extra := 0;
               for J in Transaction_Id_Type range (I+1)..Max_Transactions
               loop
                  exit when My_System(J).Transaction_Id /=
                    My_System(I).Transaction_Id;
                  Extra := Extra+1;
               end loop;

               Max_D := Large_Time;
               for K in reverse 0..Extra
               loop
                  for J in reverse 1..My_System(I+K).Ni
                  loop
                     -- Mark the actions with deadlines to calculate factors
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        if My_System(I+K).The_Task(J).Dij /= Large_Time and
                          My_System(I+K).The_Task(J).Dij < Max_D then
                           --Put_Line("Calcula Factor");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             True;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Dij;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             My_System(I+K).The_Task(J).Dij;
                           Max_D := My_System(I+K).The_Task(J).Dij;
                        else
                           --Put_Line("No Calcula Factor");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             False;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             Large_Time;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             Large_Time;
                        end if;
                     elsif My_System(I+K).The_Task(J).Pav.Preassigned then
                        --Put_Line("entra preassigned 1");
                        if My_System(I+K).The_Task(J).Dij /= Large_Time and
                          My_System(I+K).The_Task(J).Dij < Max_D then
                           --Put_Line("entra preassigned 2");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             True;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Dij;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             My_System(I+K).The_Task(J).Dij;
                           Max_D := My_System(I+K).The_Task(J).Dij;
                        else
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             False;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             Large_Time;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             Large_Time;
                        end if;
                     end if;
                  end loop;
               end loop;

               --Preassigned Deadlines
               Sum_D := 0.0;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     if My_System(I+K).The_Task(J).Pav.Preassigned then
                        Sum_D := My_System(I+K).The_Task(J).SDij + Sum_D;
                     end if;

                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        My_System(I+K).The_Task(J).Pav.D_0 :=
                          My_System(I+K).The_Task(J).Pav.D_0 - Sum_D;
                        My_System(I+K).The_Task(J).Pav.D_I :=
                          My_System(I+K).The_Task(J).Pav.D_0;
                        Sum_D := 0.0;
                     end if;
                  end loop;
               end loop;

               --Factors
               Sum_C := 0.0;
               Assigned_D := 0.0;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     -- Calculate factors
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        Sum_C := Sum_C+My_System(I+K).The_Task(J).Cijown;
                     end if;
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        My_System(I+K).The_Task(J).Pav.Factor :=
                          (My_System(I+K).The_Task(J).Pav.D_0-Assigned_D)/
                          Sum_C;
                        --Put_Line("Mostrar Factors");
                        --Put_Line(Time'Image
                        --         (My_System(I+K).The_Task(J).Pav.Factor));
                        Sum_C := 0.0;
                        Assigned_D := My_System(I+K).The_Task(J).Pav.D_0;
                     else
                        My_System(I+K).The_Task(J).Pav.Factor := 1.0;
                     end if;
                     --end if;
                  end loop;
               end loop;

               Factor := Max_Factor;
               for K in reverse 0..Extra
               loop
                  for J in reverse 1..My_System(I+K).Ni
                  loop
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        Factor := My_System(I+K).The_Task(J).Pav.Factor;
                     end if;
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        if Factor = Max_Factor then
                           My_System(I+K).The_Task(J).Pav.D_0 := Large_Time;
                        else
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Cijown*Factor;
                        end if;
                     end if;
                  end loop;
               end loop;
               exit when (I+Extra+1) not in Transaction_Id_Type;
               I := I+Extra+1;
            end loop;
         end;


         -- Save Deadlines Pav.D_0 -> SDij

         Save_Deadlines;

         -- Fix SDij when necessary

--           declare
--
--              t : Time;
--              cambios : boolean := false;
--              P : Processor_ID_Type;
--
--              function tocar_deadline (I : Transaction_ID_Type;
--                                       J : Task_ID_Type;
--                                       t : Time;
--                                       P : Processor_ID_Type) return Boolean is
--                 b : Transaction_ID_Type;
--              begin
--                 b := 1;
--                 loop
--                    for c in 1..My_System(I).Ni
--                    loop
--                       if b/=I or c/=J then
--                          if ((My_System(b).The_Task(c).Procij = P) and
--                                (My_System(b).The_Task(c).SDij = t)) then
--                             return true;
--                          end if;
--                       end if;
--                    end loop;
--                    --New_line;
--                    exit when (b+1) not in Transaction_ID_Type;
--                    b := b + 1;
--                 end loop;
--                 return false;
--              end tocar_deadline;
--
--              procedure mostrar_sistema is
--                 I : Transaction_Id_Type := 1;
--              begin
--                 loop
--                    Put(Transaction_Id_Type'Image(I)&" - ");
--                    for J in 1..My_System(I).Ni
--                    loop
--                       Put("(");
--                       Put(Integer(My_System(I).The_Task(J).Procij),1);
--                       Put(":");
--                       Put(Float(My_System(I).The_Task(J).SDij),2,2,0);
--                       Put(") ");
--                    end loop;
--                    New_line;
--                    exit when (I+1) not in Transaction_ID_Type;
--                    I := I + 1;
--                 end loop;
--              end mostrar_sistema;
--
--           begin
--              loop
--                 for I in Transaction_ID_Type range 1..Max_Transactions loop
--                    exit when My_System(I).Ni = 0;
--                    for J in 1..My_System(I).Ni loop
--                       P := My_System(I).The_Task(J).Procij;
--                       t := My_System(I).The_Task(J).SDij;
--                       if tocar_deadline(I,J,t,P) then
--                          Put("Changes in : ");
--                          Put(Integer(I),2);Put(",");
--                          Put(Integer(J),2); New_Line;
--                          if My_System(I).Ni = 1 then
--                             My_System(I).The_Task(J).SDij :=
--                               My_System(I).The_Task(J).SDij + 1.0;
--                          else
--                             if J = My_System(I).Ni then
--                                My_System(I).The_Task(J-1).SDij :=
--                                  My_System(I).The_Task(J-1).SDij - 1.0;
--                                My_System(I).The_Task(J).SDij :=
--                                  My_System(I).The_Task(J).SDij + 1.0;
--                             else
--                                My_System(I).The_Task(J+1).SDij :=
--                                  My_System(I).The_Task(J+1).SDij - 1.0;
--                                My_System(I).The_Task(J).SDij :=
--                                  My_System(I).The_Task(J).SDij + 1.0;
--                             end if;
--                          end if;
--                          cambios := true;
--                       end if;
--                    end loop;
--                 end loop;
--                 exit when not(cambios);
--                 cambios := false;
--              end loop;
--           end;

         --Translation.Translate_Priorities(My_System,The_System);

         --Put_line("Translate deadlines...");
         Translation.Translate_Deadlines(My_System,The_System);

         --Show deadlines just for tests
         --Put_Line("Mostrando deadlines de inicializacion...");
         --Translation.Show_Deadlines(My_System,The_System);

         Save_Pavs;

      end Initialize_Deadlines;

      --------------------------
      -- Assign_New_Deadlines --
      --------------------------

      procedure Assign_New_Deadlines is

         -- Assignes deadlines according to local deadlines calculated

         type Excess_Type is record
            Excess : Time := 0.0;
            Less, Plus : Time := 0.0;
         end record;

         Excess_For_Processors : array (Processor_ID_Type) of Excess_Type;
         Max_Excess_For_Events : array (Transaction_ID_Type) of Excess_Type;
         Max_Excess_For_Processors : Time := 0.0;

         Sum_D,Sum_C,Assigned_D,Factor : Time;


      begin

         Translation.Translate_Linear_System_With_Results
           (The_System,My_System,My_Verbose);
         Restore_Pavs;

         --Cambio de Jitter de salida por Jitters de entrada (Jij = Ri-1)

         declare
            I : Transaction_Id_type;
            Extra : Transaction_Id_Type;
         begin
            I := 1;
            while My_System(I).Ni/=0
            loop
               -- Calculate Extra
               Extra := 0;
               for J in Transaction_Id_Type range (I+1)..Max_Transactions
               loop
                  exit when My_System(J).Transaction_Id /=
                    My_System(I).Transaction_Id;
                  Extra := Extra+1;
               end loop;

               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     if k = 0 and J = 1 then
                        My_System(I+K).The_Task(J).Jij := Time(0);
                     else
                        if J = 0 and K > 0 then
                           My_System(I+K).The_Task(J).Jij :=
                             My_System(I+K-1).The_Task(My_System(I+K-1).Ni).Rij;
                        else
                           My_System(I+K).The_Task(J).Jij :=
                             My_System(I+K).The_Task(J-1).Rij;
                        end if;
                     end if;
                  end loop;
               end loop;

               exit when (I+Extra+1) not in Transaction_Id_Type;
               I := I+Extra+1;
            end loop;
         end;

         --Translation.Show_Linear_Translation(My_System);

         -- Excesses for actions

         declare
            I : Transaction_Id_Type;
            Extra : Transaction_Id_Type;
            -- Number of extra consecutive transactions with the same ID
         begin

            I := 1;
            while My_System(I).Ni/=0
            loop

               -- Calculate Extra
               Extra := 0;
               for J in Transaction_Id_Type range (I+1)..Max_Transactions
               loop
                  exit when My_System(J).Transaction_Id /=
                    My_System(I).Transaction_Id;
                  Extra := Extra+1;
               end loop;

               Assigned_D := 0.0;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     -- Calculate factors
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        My_System(I+K).The_Task(J).Pav.Factor :=
                          My_System(I+K).The_Task(J).Rij/
                          (My_System(I+K).The_Task(J).Pav.D_I-Assigned_D);
                        Assigned_D := My_System(I+K).The_Task(J).Pav.D_I;
                     end if;
                  end loop;
               end loop;

               Factor := 1.0;
               for K in reverse 0..Extra
               loop
                  for J in reverse 1..My_System(I+K).Ni
                  loop
                     -- Calculate excesses
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        Factor := My_System(I+K).The_Task(J).Pav.Factor;
                     end if;

                     --if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                     --not My_System(I).The_Task(J).Pav.Is_Polling then

                     --if My_System(I+K).The_Task(J).Pav.D_0 >
                     --My_System(I+K).The_Task(J).Tij then
                     --Put_line("dij > Ti");
                     My_System(I+K).The_Task(J).Pav.Excess :=
                       (My_System(I+K).The_Task(J).Rij -
                          My_System(I+K).The_Task(J).Pav.D_0)*Factor;
                     --else
                     --Put_line("dij < Ti");
                     --My_System(I+K).The_Task(J).Pav.Excess :=
                     --(My_System(I+K).The_Task(J).Rij
                     --- My_System(I+K).The_Task(J).Jij
                     --- My_System(I+K).The_Task(J).Pav.D_0)*Factor;
                     --end if;

                     if False then
                        New_Line;
                        Put("[");
                        Put(Integer(I),2);Put(",");
                        Put(Integer(J),2);Put("]");New_Line;
                        Put("D_0 : ");
                        Put(Integer(My_System(I+K).The_Task(J).Pav.D_0));
                        New_Line;
                        Put("Tij : ");
                        Put(Integer(My_System(I+K).The_Task(J).Tij));
                        New_Line;
                        Put("Rij : ");
                        Put(Integer(My_System(I+K).The_Task(J).Rij));
                        New_Line;
                        Put("Jij : ");
                        Put(Integer(My_System(I+K).The_Task(J).Jij));
                        New_Line;
                        Put("Procij : ");
                        Put(Integer(My_System(I+K).The_Task(J).Procij));
                        New_Line;
                        Put("Cijown : ");
                        Put(Integer(My_System(I+K).The_Task(J).Cijown));
                        New_Line;
                     end if;

                     --end if;
                  end loop;
               end loop;
               exit when (I+Extra+1) not in Transaction_Id_Type;
               I := I+Extra+1;
            end loop;
         end;

         -- Maximum excesses for events and processing resources

         declare
            I, T_Id : Transaction_Id_Type;
            Extra : Transaction_Id_Type;
            -- Number of extra consecutive transactions with the same ID
         begin

            I := 1;
            while My_System(I).Ni/=0
            loop

               -- Calculate Extra
               Extra := 0;
               for J in Transaction_Id_Type range (I+1)..Max_Transactions
               loop
                  exit when My_System(J).Transaction_Id /=
                    My_System(I).Transaction_Id;
                  Extra := Extra+1;
               end loop;
               T_Id := My_System(I).Transaction_Id;
               --Put("T_ID : "&Transaction_Id_Type'Image(T_Id));New_line;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     --if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                     --not My_System(I).The_Task(J).Pav.Is_Polling then
                     if My_System(I+K).The_Task(J).Pav.Excess > 0.0 then
                        Excess_For_Processors
                          (My_System(I+K).The_Task(J).Procij).Plus :=
                          Excess_For_Processors
                            (My_System(I+K).The_Task(J).Procij).Plus+
                          My_System(I+K).The_Task(J).Pav.Excess;
                        Max_Excess_For_Events(T_Id).Plus :=
                          Max_Excess_For_Events(T_Id).Plus+
                          My_System(I+K).The_Task(J).Pav.Excess;
                     else
                        Excess_For_Processors
                          (My_System(I+K).The_Task(J).Procij).Less :=
                          Excess_For_Processors
                            (My_System(I+K).The_Task(J).Procij).Less+
                          My_System(I+K).The_Task(J).Pav.Excess;
                        Max_Excess_For_Events(T_Id).Less :=
                          Max_Excess_For_Events(T_Id).Less+
                          My_System(I+K).The_Task(J).Pav.Excess;
                     end if;
                     --New
                     if abs(My_System(I+K).The_Task(J).Pav.Excess) >
                       Max_Excess_For_Events(T_Id).Excess then
                        Max_Excess_For_Events(T_Id).Excess :=
                          abs(My_System(I+K).The_Task(J).Pav.Excess);
                     end if;
                     --end if;
                  end loop;
               end loop;

               if False then
                  --Put("T_ID : "&Transaction_Id_Type'Image(T_Id));
                  Put("Less : "&Time'Image(Max_Excess_For_Events(T_Id).Less));
                  New_line;
                  Put_Line("Plus : "&
                           Time'Image(Max_Excess_For_Events(T_Id).Plus));
               end if;

               exit when (I+Extra+1) not in Transaction_Id_Type;

               I := I+Extra+1;

            end loop;
         end;

         for I in Processor_Id_Type range 1..Max_Processors
         loop
            Excess_For_Processors(I).Excess := Excess_For_Processors(I).Plus +
              Excess_For_Processors(I).Less;
            if abs(Excess_For_Processors(I).Excess) >
              Max_Excess_For_Processors then
               Max_Excess_For_Processors:= abs(Excess_For_Processors(I).Excess);
            end if;
         end loop;

         -- Deadline assignment

         for I in Transaction_Id_Type range 1..Max_Transactions
         loop
            exit when My_System(I).Ni=0;
            declare
               T_Id : Transaction_Id_Type;
            begin
               T_Id :=My_System(I).Transaction_Id;
               for J in 1..My_System(I).Ni
               loop
                  if not My_System(I).The_Task(J).Pav.Preassigned and then
                    not My_System(I).The_Task(J).Pav.Is_Polling then
                     if My_System(I).The_Task(J).Pav.D_0 /= Large_Time then
                        My_System(I).The_Task(J).Pav.D_0 :=
                          My_System(I).The_Task(J).Pav.D_0*
                          (1.0+
                             Excess_For_Processors
                               (My_System(I).The_Task(J).Procij).Excess/
                           (Time(Kr)*Max_Excess_For_Processors))*
                          (1.0+
                             My_System(I).The_Task(J).Pav.Excess/
                           (Time(Ka)*Max_Excess_For_Events(T_Id).Excess));
                     end if;
                  end if;
               end loop;
            end;
         end loop;

         -- Adjustment of assigned deadlines to real deadlines

         declare
            I : Transaction_Id_Type;
            Extra : Transaction_Id_Type;

            -- Number of extra consecutive transactions with the same ID
         begin

            I := 1;
            while My_System(I).Ni/=0
            loop

               -- Calculate Extra
               Extra := 0;
               for J in Transaction_Id_Type range (I+1)..Max_Transactions
               loop
                  exit when My_System(J).Transaction_Id /=
                    My_System(I).Transaction_Id;
                  Extra := Extra+1;
               end loop;

               Assigned_D := 0.0;
               Sum_D := 0.0;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     -- Calculate factors
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        Sum_D := Sum_D+My_System(I+K).The_Task(J).Pav.D_0;
                     end if;
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        My_System(I+K).The_Task(J).Pav.Factor :=
                          (My_System(I+K).The_Task(J).Pav.D_I-Assigned_D)/
                          Sum_D;
                        Sum_D := 0.0;
                        Assigned_D := My_System(I+K).The_Task(J).Pav.D_I;
                     end if;
                  end loop;
               end loop;

               Factor := 1.0;
               for K in reverse 0..Extra
               loop
                  for J in reverse 1..My_System(I+K).Ni
                  loop
                     -- Adjust deadlines
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        Factor := My_System(I+K).The_Task(J).Pav.Factor;
                     end if;

                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        My_System(I+K).The_Task(J).Pav.D_0 :=
                          My_System(I+K).The_Task(J).Pav.D_0*Factor;
                     end if;
                  end loop;
               end loop;
               exit when (I+Extra+1) not in Transaction_Id_Type;
               I := I+Extra+1;
            end loop;
         end;

         -- Save New Deadlines Pav.D_0 -> SDij
         Save_Deadlines;

         -- Fix SDij when needed (to avoid two or more tasks with the same SDij at the same CPU)
--           declare
--
--              t : Time;
--              cambios : boolean := false;
--              P : Processor_ID_Type;
--
--              function tocar_deadline (I : Transaction_ID_Type;
--                                       J : Task_ID_Type;
--                                       t : Time;
--                                       P : Processor_ID_Type) return Boolean is
--                 b : Transaction_ID_Type;
--              begin
--                 b := 1;
--                 loop
--                    for c in 1..My_System(I).Ni
--                    loop
--                       if b/=I or c/=J then
--                          if ((My_System(b).The_Task(c).Procij = P) and
--                                (My_System(b).The_Task(c).SDij = t)) then
--                             return true; end if;
--                       end if;
--                    end loop;
--                    exit when (b+1) not in Transaction_ID_Type;
--                    b := b + 1;
--                 end loop;
--                 return false;
--              end tocar_deadline;
--
--              procedure mostrar_sistema is
--                 I : Transaction_Id_Type := 1;
--              begin
--                 loop
--                    Put(Transaction_Id_Type'Image(I)&" - ");
--                    for J in 1..My_System(I).Ni
--                    loop
--                       Put("(");
--                       Put(Integer(My_System(I).The_Task(J).Procij),2);
--                       Put(":");
--                       Put(Float(My_System(I).The_Task(J).SDij),5,2,0);
--                       Put(") ");
--                    end loop;
--                    New_line;
--                    exit when (I+1) not in Transaction_ID_Type;
--                    I := I + 1;
--                 end loop;
--              end mostrar_sistema;
--
--           begin
--              loop
--                 for I in Transaction_ID_Type range 1..Max_Transactions loop
--                    exit when My_System(I).Ni = 0;
--                    for J in 1..My_System(I).Ni loop
--                       P := My_System(I).The_Task(J).Procij;
--                       t := My_System(I).The_Task(J).SDij;
--                       if tocar_deadline(I,J,t,P) then
--                          Put("Cambios en : ");
--                          Put(Integer(I),2);Put(",");
--                          Put(Integer(J),2); New_Line;
--                          if My_System(I).Ni = 1 then
--                             My_System(I).The_Task(J).SDij :=
--                               My_System(I).The_Task(J).SDij + 1.0;
--                          else
--                             if J = My_System(I).Ni then
--                                My_System(I).The_Task(J-1).SDij :=
--                                  My_System(I).The_Task(J-1).SDij - 1.0;
--                                My_System(I).The_Task(J).SDij :=
--                                  My_System(I).The_Task(J).SDij + 1.0;
--                             else
--                                My_System(I).The_Task(J+1).SDij :=
--                                  My_System(I).The_Task(J+1).SDij - 1.0;
--                                My_System(I).The_Task(J).SDij :=
--                                  My_System(I).The_Task(J).SDij + 1.0;
--                             end if;
--                          end if;
--                       end if;
--                    end loop;
--                 end loop;
--                 exit when not(cambios);
--                 cambios := false;
--              end loop;
--           end;

         Translation.Translate_Deadlines(My_System,The_System);
         --Put_Line("Showing deadlines...");
         --Translation.Show_Deadlines(My_System,The_System);

         Save_Pavs;

      end Assign_New_Deadlines;


      ----------------------------
      -- Save_Optimum_Deadlines --
      ----------------------------

      procedure Save_Optimum_Deadlines is

      begin
         Mast.Tools.Schedulability_Index.Calculate_Schedulability_Index
           (The_System,Sched_Index,Verbose);
         --Put("Schedulability Index :");
         Mast.Tools.Schedulability_Index.Print(Sched_Index);
         if Sched_Index > Optimum_Sched_Index then
            Optimum_Sched_Index := Sched_Index;
            if Verbose then
               Put("Saving optimum deadline assignment with "&
                   "schedulability index : ");
               Mast.Tools.Schedulability_Index.Print(Optimum_Sched_Index);
            end if;
            for I in Transaction_Id_Type range 1..Max_Transactions
            loop
               exit when My_System(I).Ni=0;
               for J in 1..My_System(I).Ni
               loop
                  if not My_System(I).The_Task(J).Pav.Preassigned and then
                    not My_System(I).The_Task(J).Pav.Is_Polling then
                     --Pav.D_1 as Pav.Optimum_Deadline in HOPA
                     My_System(I).The_Task(J).Pav.D_1 :=
                       My_System(I).The_Task(J).SDij;
                     Pavs(I)(J).D_1 :=
                       My_System(I).The_Task(J).SDij;
                  end if;
               end loop;
            end loop;
         end if;

      exception
         when Mast.Tools.Schedulability_Index.Inconclusive =>
            null;

      end Save_Optimum_Deadlines;

      -------------------------------
      -- Restore_Optimum_Deadlines --
      -------------------------------

      procedure Restore_Optimum_Deadlines is

      begin
         if Overall_Schedulable then
            if Verbose then
               Put("Restoring optimum deadline assignment with "&
                   "schedulability index : ");
               Mast.Tools.Schedulability_Index.Print(Optimum_Sched_Index);
            end if;
            for I in Transaction_Id_Type range 1..Max_Transactions
            loop
               exit when My_System(I).Ni=0;
               for J in 1..My_System(I).Ni
               loop
                  if not My_System(I).The_Task(J).Pav.Preassigned and then
                    not My_System(I).The_Task(J).Pav.Is_Polling then
                     My_System(I).The_Task(J).SDij :=
                       My_System(I).The_Task(J).Pav.D_1;
                  end if;
               end loop;
            end loop;

            Translation.Translate_Deadlines(My_System,The_System);

         else
            for I in Transaction_Id_Type range 1..Max_Transactions
            loop
               exit when My_System(I).Ni=0;
               for J in 1..My_System(I).Ni
               loop
                  My_System(I).The_Task(J).Prioij :=
                    My_System(I).The_Task(J).Pav.Preassigned_Prio;
               end loop;
            end loop;

            Translation.Translate_Deadlines(My_System,The_System);

         end if;
      end Restore_Optimum_Deadlines;

      --------------------
      -- Stop_Condition --
      --------------------

      function Stop_Condition return Boolean is

      begin
         if Optimize then
            if Schedulable then
               Save_Optimum_Deadlines;
               Overall_Schedulable := True;
               if not Optimizing then
                  Optimizing := True;
               else
                  Overiteration := Overiteration-1;
                  if Overiteration <= 0 then
                     return True;
                  end if;
               end if;
            else
               if Optimizing then
                  Overiteration := Overiteration-1;
                  if Overiteration <= 0 then
                     return True;
                  end if;
               end if;
            end if;
         else
            if Schedulable then
               Save_Optimum_Deadlines;
               Overall_Schedulable := True;
               return True;
            end if;
         end if;
         --if (Float(Counter_Of_Aborted_Analysis)/
         --Float(Counter_Of_Iterations)) > 0.5 then
         --   return True;
         --end if;
         return False;
      end Stop_Condition;


   begin

      Overiteration := HOSPA_Parameters.Get_Overiterations;
      Optimize := (HOSPA_Parameters.Get_Overiterations /= 0);

      Initialize_Deadlines;
      -- Initial values according to deadline monotonic

      HOSPA_Parameters.Rewind_Iterations_List;

      for I in 1..HOSPA_Parameters.Size_Of_Iterations_List
      loop
         HOSPA_Parameters.Rewind_K_List;
         Max_Iter := HOSPA_Parameters.Get_Next_Iterations;
         --Initialize_Deadlines;
         for J in 1..HOSPA_Parameters.Size_Of_K_List
         loop
            K := Hospa_Parameters.Get_Next_K_Pair;
            Ka := HOSPA_Parameters.Get_Ka(K);
            Kr := HOSPA_Parameters.Get_Kr(K);
            for L in 1..Max_Iter
            loop
               Mast.Tools.Calculate_Ceilings_And_Levels(The_System,My_Verbose);
               declare
                  Aborted : Boolean := False;
               begin
                  My_Back_Up_System := My_System;
                  Counter_Of_Iterations :=
                    Integer'Succ(Counter_Of_Iterations);
                  if False then
                     Put("Starting analysis on iteration: "&
                         Integer'Image(Counter_Of_Iterations));
                     Put("Ka : ");Put(Float'Image(Ka));
                     Put("Kr : ");Put(Float'Image(Kr));
                     New_line;
                  end if;
                  if ATC_Enabled then
                     select
                        delay Analysis_Stop_Time;
                        My_System := My_Back_Up_System;
                        Aborted := True;
                        Counter_Of_Aborted_Analysis :=
                          Integer'Succ(Counter_Of_Aborted_Analysis);
                     then abort
                        -- Analysis Tool
                        The_Tool(The_System,My_Verbose);
                     end select;
                  else
                     Aborted:=False;
                     -- Analysis Tool
                     The_Tool(The_System,My_Verbose);
                  end if;
                  if False then
                     if Aborted then
                        Put_Line("    -> Analysis aborted.");
                     else
                        Put_Line("    -> Analysis finished.");
                     end if;
                  end if;
                  if Aborted then
                     Schedulable := False;
                  else
                     Mast.Tools.Check_System_Schedulability
                       (The_System,Schedulable,My_Verbose);
                  end if;
                  Stop_Algorithm := Stop_Condition;
                  exit when Stop_Algorithm or
                    ((I = HOSPA_Parameters.Size_Of_Iterations_List) and
                       (J = HOSPA_Parameters.Size_Of_K_List) and
                       (L = Max_Iter));
                  -- New deadline assignment
                  if not Aborted then
                     Assign_New_Deadlines;
                  else
                     null;
                  end if;
               exception
                  when CONSTRAINT_ERROR =>
                     Put_Line("HOSDA EDF : CONSTRAINT_ERROR");
                     Counter_Of_Aborted_Analysis :=
                       Integer'Succ(Counter_Of_Aborted_Analysis);
                     if Counter_Of_Aborted_Analysis > 5 then
                        Put_Line("Reached maximum number of aborted analysis...");
                        raise;
                     end if;
               end;
            end loop;
            exit when Stop_Algorithm;
         end loop;
         exit when Stop_Algorithm;
      end loop;

      Restore_Optimum_Deadlines;

      if Verbose then
         Put_Line("Linear HOSDA statistics:");
         Put_Line("   Total number of iterations ------------------------> "&
                  Integer'Image(Counter_Of_Iterations));
         if Overall_Schedulable then
            Put_Line("The deadline assignment process "&
                     "makes the system schedulable");
         else
            Put_Line("The deadline assignment process "&
                     "does not make the system schedulable");
         end if;

      end if;

   end Local_HOSDA;

   --------------------------------------------
   -- Proportional_Local_Deadline_Assignment --
   --------------------------------------------

   procedure Proportional_Local_Deadline_Assignment
     (The_System : in out MAST.Systems.System;
      The_Tool : in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose : in Boolean:=False) is

      Schedulable : Boolean := False;

      Sched_Index : Mast.Tools.Schedulability_Index.Index:=
        Mast.Tools.Schedulability_Index.Lower_Index;

      Max_Processors:constant Processor_ID:=Processor_ID
        (Processing_Resources.Lists.Size(The_System.Processing_Resources));
      Max_Transactions:constant Transaction_ID:=Transaction_ID
        ((Mast.Max_Numbers.Calculate_Max_Transactions(The_System)));
      Max_Tasks_Per_Transaction:constant Task_ID:=Task_ID
        (Mast.Max_Numbers.Calculate_Max_Tasks_Per_Transaction(The_System));

      subtype Processor_ID_Type is Processor_ID
      range 0..Max_Processors;
      subtype Transaction_ID_Type is Transaction_ID
      range 0..Max_Transactions;
      subtype Task_ID_Type is Task_ID
      range 0..Max_Tasks_Per_Transaction;

      package Translation is new Linear_Translation
        (Processor_ID_Type, Transaction_ID_Type, Task_ID_Type,
         Max_Processors, Max_Transactions, Max_Tasks_Per_Transaction);

      use Translation;

      My_System,My_Back_Up_System : Translation.Linear_Transaction_System;

      type List_For_Tasks is array
        (Task_ID_Type range 1..Max_Tasks_Per_Transaction)
      of Translation.Priority_Assignment_Vars;

      type List_For_Transactions is array
        (Transaction_ID_Type range 1..Max_Transactions)
      of List_For_Tasks;


      --------------------
      -- Save Deadlines --
      --------------------

      procedure Save_Deadlines is
      begin
         for I in Transaction_ID_Type range 1..Max_Transactions
         loop
            for J in Task_ID_Type range 1..Max_Tasks_Per_Transaction
            loop
               My_System(I).The_Task(J).SDij :=
                 My_System(I).The_Task(J).Pav.D_0;
            end loop;
         end loop;
      end Save_Deadlines;


      --------------------------
      -- Initialize_Deadlines --
      --------------------------

      procedure Initialize_Deadlines is

         -- Initialize deadlines according to WCETs

         Max_Factor : constant Time := Large_Time;
         Max_D,Sum_C,Assigned_D,Factor,Sum_D : Time;

      begin
         Translation.Translate_Linear_System(The_System,My_System,My_Verbose);

         -- Deadline assignment

         declare

            I : Transaction_Id_Type;
            Extra : Transaction_Id_Type;
            -- Number of extra consecutive transactions with the same ID
         begin

            I := 1;
            while My_System(I).Ni/=0
            loop

               -- Calculate Extra
               Extra := 0;
               for J in Transaction_Id_Type range (I+1)..Max_Transactions
               loop
                  exit when My_System(J).Transaction_Id /=
                    My_System(I).Transaction_Id;
                  Extra := Extra+1;
               end loop;


               Max_D := Large_Time;
               for K in reverse 0..Extra
               loop
                  for J in reverse 1..My_System(I+K).Ni
                  loop
                     -- Mark the actions with deadlines to calculate factors
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        if My_System(I+K).The_Task(J).Dij /= Large_Time and
                          My_System(I+K).The_Task(J).Dij < Max_D then
                           --Put_Line("Calcula Factor");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             True;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Dij;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             My_System(I+K).The_Task(J).Dij;
                           Max_D := My_System(I+K).The_Task(J).Dij;
                        else
                           --Put_Line("No Calcula Factor");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             False;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             Large_Time;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             Large_Time;
                        end if;
                     elsif My_System(I+K).The_Task(J).Pav.Preassigned then
                        --Put_Line("entra preassigned 1");
                        if My_System(I+K).The_Task(J).Dij /= Large_Time and
                          My_System(I+K).The_Task(J).Dij < Max_D then
                           --Put_Line("entra preassigned 2");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             True;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Dij;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             My_System(I+K).The_Task(J).Dij;
                           Max_D := My_System(I+K).The_Task(J).Dij;
                        else
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             False;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             Large_Time;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             Large_Time;
                        end if;
                     end if;
                  end loop;
               end loop;

               --Preassigned Deadlines
               Sum_D := 0.0;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     if My_System(I+K).The_Task(J).Pav.Preassigned then
                        Sum_D := My_System(I+K).The_Task(J).SDij + Sum_D;
                     end if;

                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        My_System(I+K).The_Task(J).Pav.D_0 :=
                          My_System(I+K).The_Task(J).Pav.D_0 - Sum_D;
                        My_System(I+K).The_Task(J).Pav.D_I :=
                          My_System(I+K).The_Task(J).Pav.D_0;
                        Sum_D := 0.0;
                     end if;
                  end loop;
               end loop;


               Sum_C := 0.0;
               Assigned_D := 0.0;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     -- Calculate factors
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        Sum_C := Sum_C+My_System(I+K).The_Task(J).Cijown;
                     end if;
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        My_System(I+K).The_Task(J).Pav.Factor :=
                          (My_System(I+K).The_Task(J).Pav.D_0-Assigned_D)/
                          Sum_C;
                        --Put_Line("Mostrar Factors");
                        --Put_Line(
                        --         Time'Image(My_System(I+K).The_Task(J).Pav.Factor));
                        Sum_C := 0.0;
                        Assigned_D := My_System(I+K).The_Task(J).Pav.D_0;
                     else
                        My_System(I+K).The_Task(J).Pav.Factor := 1.0;
                     end if;
                  end loop;
               end loop;

               Factor := Max_Factor;
               for K in reverse 0..Extra
               loop
                  for J in reverse 1..My_System(I+K).Ni
                  loop
                     -- Calculate deadlines
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        Factor := My_System(I+K).The_Task(J).Pav.Factor;
                     end if;
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        if Factor = Max_Factor then
                           My_System(I+K).The_Task(J).Pav.D_0 := Large_Time;
                        else
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Cijown*Factor;
                        end if;
                     end if;
                  end loop;
               end loop;
               exit when (I+Extra+1) not in Transaction_Id_Type;
               I := I+Extra+1;

            end loop;

         end;

         -- Save Deadlines Pav.D_0 -> SDij
         Save_Deadlines;

--           declare
--
--              t : Time;
--              cambios : boolean := false;
--              P : Processor_ID_Type;
--
--              function tocar_deadline (I : Transaction_ID_Type;
--                                       J : Task_ID_Type;
--                                       t : Time;
--                                       P : Processor_ID_Type) return Boolean is
--                 b : Transaction_ID_Type;
--              begin
--                 b := 1;
--                 loop
--                    for c in 1..My_System(I).Ni
--                    loop
--                       if b/=I or c/=J then
--                          if ((My_System(b).The_Task(c).Procij = P) and
--                                (My_System(b).The_Task(c).SDij = t)) then
--                             return true;
--                          end if;
--                       end if;
--                    end loop;
--                    exit when (b+1) not in Transaction_ID_Type;
--                    b := b + 1;
--                 end loop;
--                 return false;
--              end tocar_deadline;
--
--              procedure mostrar_sistema is
--                 I : Transaction_Id_Type := 1;
--              begin
--                 loop
--                    for J in 1..My_System(I).Ni
--                    loop
--                       Put("(");Put(Integer(My_System(I).The_Task(J).Procij),1);
--                       Put(":");Put(Float(My_System(I).The_Task(J).SDij),2,2,0);
--                       Put(") ");
--                    end loop;
--                    exit when (I+1) not in Transaction_ID_Type;
--                    I := I + 1;
--                 end loop;
--              end mostrar_sistema;
--
--           begin
--              loop
--                 for I in Transaction_ID_Type range 1..Max_Transactions loop
--                    exit when My_System(I).Ni = 0;
--                    for J in 1..My_System(I).Ni loop
--                       P := My_System(I).The_Task(J).Procij;
--                       t := My_System(I).The_Task(J).SDij;
--                       if tocar_deadline(I,J,t,P) then
--                          Put("Deadline corrections in : ");
--                          Put(Integer(I),2);Put(",");
--                          Put(Integer(J),2); New_Line;
--                          if My_System(I).Ni = 1 then
--                             My_System(I).The_Task(J).SDij :=
--                               My_System(I).The_Task(J).SDij + 1.0;
--                          else
--                             if J = My_System(I).Ni then
--                                My_System(I).The_Task(J-1).SDij :=
--                                  My_System(I).The_Task(J-1).SDij - 1.0;
--                                My_System(I).The_Task(J).SDij :=
--                                  My_System(I).The_Task(J).SDij + 1.0;
--                             else
--                                My_System(I).The_Task(J+1).SDij :=
--                                  My_System(I).The_Task(J+1).SDij - 1.0;
--                                My_System(I).The_Task(J).SDij :=
--                                  My_System(I).The_Task(J).SDij + 1.0;
--                             end if;
--                          end if;
--                          cambios := true;
--                       end if;
--                    end loop;
--                 end loop;
--                 exit when not(cambios);
--                 cambios := false;
--              end loop;
--           end;

         Translation.Translate_Deadlines(My_System,The_System);

      end Initialize_Deadlines;

   begin

      Initialize_Deadlines;
      Mast.Tools.Calculate_Ceilings_And_Levels(The_System,My_Verbose);

      The_Tool(The_System,My_Verbose);

      Mast.Tools.Check_System_Schedulability(The_System,Schedulable,My_Verbose);

      if Verbose then
         Put_Line("Proportional Deadline Assigment statistics");
         if Schedulable then
            Mast.Tools.Schedulability_Index.Calculate_Schedulability_Index
              (The_System,Sched_Index,Verbose);
            Put_Line("The Deadline Assigment "&
                     "process makes the system schedulable");
            Put("Deadline Assignment with schedulability index : ");
            Mast.Tools.Schedulability_Index.Print(Sched_Index);
         else
            Put_Line("The Deadline Assigment "&
                     "process does not make the system schedulable");
         end if;
      end if;

   end Proportional_Local_Deadline_Assignment;

   ------------------------------------------
   -- Normalized_Local_Deadline_Assignment --
   ------------------------------------------

   procedure Normalized_Local_Deadline_Assignment
     (The_System : in out MAST.Systems.System;
      The_Tool : in Mast.Tools.Worst_Case_Analysis_Tool;
      Verbose : in Boolean:=false) is

      Schedulable : Boolean := False;

      Sched_Index : Mast.Tools.Schedulability_Index.Index:=
        Mast.Tools.Schedulability_Index.Lower_Index;

      Max_Processors:constant Processor_ID:=Processor_ID
        (Processing_Resources.Lists.Size(The_System.Processing_Resources));
      Max_Transactions:constant Transaction_ID:=Transaction_ID
        ((Mast.Max_Numbers.Calculate_Max_Transactions(The_System)));
      Max_Tasks_Per_Transaction:constant Task_ID:=Task_ID
        (Mast.Max_Numbers.Calculate_Max_Tasks_Per_Transaction(The_System));

      subtype Processor_ID_Type is Processor_ID
      range 0..Max_Processors;
      subtype Transaction_ID_Type is Transaction_ID
      range 0..Max_Transactions;
      subtype Task_ID_Type is Task_ID
      range 0..Max_Tasks_Per_Transaction;

      package Translation is new Linear_Translation
        (Processor_ID_Type, Transaction_ID_Type, Task_ID_Type,
         Max_Processors, Max_Transactions, Max_Tasks_Per_Transaction);

      use Translation;

      My_System,My_Back_Up_System : Translation.Linear_Transaction_System;

      type List_For_Tasks is array
        (Task_ID_Type range 1..Max_Tasks_Per_Transaction)
      of Translation.Priority_Assignment_Vars;

      type List_For_Transactions is array
        (Transaction_ID_Type range 1..Max_Transactions)
      of List_For_Tasks;


      --------------------
      -- Save Deadlines --
      --------------------

      procedure Save_Deadlines is
      begin
         for I in Transaction_ID_Type range 1..Max_Transactions
         loop
            for J in Task_ID_Type range 1..Max_Tasks_Per_Transaction
            loop
               My_System(I).The_Task(J).SDij :=
                 My_System(I).The_Task(J).Pav.D_0;
            end loop;
         end loop;
      end Save_Deadlines;


      --------------------------
      -- Initialize_Deadlines --
      --------------------------

      procedure Initialize_Deadlines is

         -- Initialize deadlines according to WCETs

         Max_Factor : constant Time := Large_Time;
         Max_D,Sum_C,Assigned_D,Factor,Sum_D : Time;

      begin
         Translation.Translate_Linear_System(The_System,My_System,My_Verbose);

         -- Deadline assignment

         declare

            I : Transaction_Id_Type;
            Extra : Transaction_Id_Type;
            -- Number of extra consecutive transactions with the same ID
         begin

            I := 1;
            while My_System(I).Ni/=0
            loop

               -- Calculate Extra
               Extra := 0;
               for J in Transaction_Id_Type range (I+1)..Max_Transactions
               loop
                  exit when My_System(J).Transaction_Id /=
                    My_System(I).Transaction_Id;
                  Extra := Extra+1;
               end loop;


               Max_D := Large_Time;
               for K in reverse 0..Extra
               loop
                  for J in reverse 1..My_System(I+K).Ni
                  loop
                     -- Mark the actions with deadlines to calculate factors
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        if My_System(I+K).The_Task(J).Dij /= Large_Time and
                          My_System(I+K).The_Task(J).Dij < Max_D then
                           --Put_Line("Calcula Factor");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             True;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Dij;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             My_System(I+K).The_Task(J).Dij;
                           Max_D := My_System(I+K).The_Task(J).Dij;
                        else
                           --Put_Line("No Calcula Factor");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             False;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             Large_Time;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             Large_Time;
                        end if;
                     elsif My_System(I+K).The_Task(J).Pav.Preassigned then
                        --Put_Line("entra preassigned 1");
                        if My_System(I+K).The_Task(J).Dij /= Large_Time and
                          My_System(I+K).The_Task(J).Dij < Max_D then
                           --Put_Line("entra preassigned 2");
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             True;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Dij;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             My_System(I+K).The_Task(J).Dij;
                           Max_D := My_System(I+K).The_Task(J).Dij;
                        else
                           My_System(I+K).The_Task(J).Pav.Calculate_Factor :=
                             False;
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             Large_Time;
                           My_System(I+K).The_Task(J).Pav.D_I :=
                             Large_Time;
                        end if;
                     end if;
                  end loop;
               end loop;


               --Preassigned Deadlines
               Sum_D := 0.0;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     if My_System(I+K).The_Task(J).Pav.Preassigned then
                        Sum_D := My_System(I+K).The_Task(J).SDij + Sum_D;
                     end if;

                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        My_System(I+K).The_Task(J).Pav.D_0 :=
                          My_System(I+K).The_Task(J).Pav.D_0 - Sum_D;
                        My_System(I+K).The_Task(J).Pav.D_I :=
                          My_System(I+K).The_Task(J).Pav.D_0;
                        Sum_D := 0.0;
                     end if;
                  end loop;
               end loop;


               Sum_C := 0.0;
               Assigned_D := 0.0;
               for K in 0..Extra
               loop
                  for J in 1..My_System(I+K).Ni
                  loop
                     -- Calculate factors
                     if not My_System(I+K).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        Sum_C := Sum_C+My_System(I+K).The_Task(J).Cijown*
                          Time
                            (Mast.Tools.Calculate_Processing_Resource_Utilization
                                 (The_System,
                                  My_System(I+K).The_Task(J).Pav.P_R_Ref,False));
                     end if;
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        My_System(I+K).The_Task(J).Pav.Factor :=
                          (My_System(I+K).The_Task(J).Pav.D_0-Assigned_D)/
                          Sum_C;
                        Sum_C := 0.0;
                        Assigned_D := My_System(I+K).The_Task(J).Pav.D_0;
                     else
                        My_System(I+K).The_Task(J).Pav.Factor := 1.0;
                     end if;
                  end loop;
               end loop;

               Factor := Max_Factor;
               for K in reverse 0..Extra
               loop
                  for J in reverse 1..My_System(I+K).Ni
                  loop
                     -- Calculate deadlines
                     if My_System(I+K).The_Task(J).Pav.Calculate_Factor then
                        Factor := My_System(I+K).The_Task(J).Pav.Factor;
                     end if;
                     if not My_System(I).The_Task(J).Pav.Preassigned and then
                       not My_System(I).The_Task(J).Pav.Is_Polling then
                        if Factor = Max_Factor then
                           My_System(I+K).The_Task(J).Pav.D_0 := Large_Time;
                        else
                           My_System(I+K).The_Task(J).Pav.D_0 :=
                             My_System(I+K).The_Task(J).Cijown*
                             Time(Mast.Tools.
                                 Calculate_Processing_Resource_Utilization
                                   (The_System,
                                    My_System(I+K).The_Task(J).Pav.P_R_Ref,
                                    True))*Factor;
                        end if;
                     end if;
                  end loop;
               end loop;
               exit when (I+Extra+1) not in Transaction_Id_Type;
               I := I+Extra+1;

            end loop;
         end;

         -- Save Deadlines Pav.D_0 -> SDij

         Save_Deadlines;

         Translation.Translate_Deadlines(My_System,The_System);

      end Initialize_Deadlines;

   begin

      Initialize_Deadlines;
      Mast.Tools.Calculate_Ceilings_And_Levels(The_System,My_Verbose);

      The_Tool(The_System,My_Verbose);

      Mast.Tools.Check_System_Schedulability(The_System,Schedulable,My_Verbose);

      if Verbose then
         Put_Line("Normalized Proportional Deadline Assigment statistics");
         if Schedulable then
            Mast.Tools.
              Schedulability_Index.
                Calculate_Schedulability_Index(The_System,Sched_Index,Verbose);
            Put_Line("The Deadline Assigment "&
                     "process makes the system schedulable");
            Put("Deadline Assignment with schedulability index : ");
            Mast.Tools.Schedulability_Index.Print(Sched_Index);
         else
            Put_Line("The Deadline Assigment "&
                     "process does not make the system schedulable");
         end if;
      end if;


   end Normalized_Local_Deadline_Assignment;

end mast.linear_local_deadline_assignment_tools;
